/********************************************************************************
 *
 * Copyright (c) 2018-2022 Advanced Micro Devices, Inc. All rights reserved.
 *
 * MIT LICENSE:
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *******************************************************************************/
#ifndef RVS_INCLUDE_RVSOPTIONS_H_
#define RVS_INCLUDE_RVSOPTIONS_H_

#define DO_PRAGMA(x) _Pragma (#x)
#define TODO(x) DO_PRAGMA(message ("TODO - " #x))

#include <string>
#include <map>

namespace rvs {

/**
 * @class options
 * @ingroup Launcher
 *
 * @brief Collection of options.
 *
 */

class options {
 public:
  static bool has_option(const std::string& Option);
  static bool has_option(std::map<std::string, std::string>& opt, const std::string& Option);
  static bool has_option(const std::string& Option, std::string* pval);
  static bool has_option(std::map<std::string, std::string>& opt, const std::string& Option, std::string* pval);
  static const std::map<std::string, std::string>& get(void);

 protected:
  //! Collection of options
  static std::map<std::string, std::string> opt;

friend class cli;
};

}  // namespace rvs


#endif  // RVS_INCLUDE_RVSOPTIONS_H_
